/*
Copyright (c) 2004,2005, Deryk Robosson <deryk@0x0a.com>
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
    * Neither the name of the Deryk Robosson nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

using System;
using System.Collections;
using System.Drawing;
using System.Windows.Forms;

namespace EvoTemplate
{
	/// <summary>
	/// Macro configuration form
	/// </summary>
	public class MacroForm : System.Windows.Forms.Form
	{
		#region Private Variables
		private System.Windows.Forms.Button buttonOkay;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.GroupBox groupBox1;
		private System.Windows.Forms.Button buttonChange;
		private System.Windows.Forms.TextBox textBoxText;
		private System.Windows.Forms.RadioButton radioButtonNone;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.ColumnHeader columnHeader1;
		private System.Windows.Forms.ColumnHeader columnHeader2;
		private System.Windows.Forms.RadioButton radioButtonAlt;
		private System.Windows.Forms.Button buttonCancel;
		private System.Windows.Forms.ListView listViewKeys;
		private System.Windows.Forms.RadioButton radioButtonCtrl;
		private System.Windows.Forms.ComboBox comboBoxKey;
		private MacroConfig macroConfig;
		private ArrayList macrosArray;
		private enum keyArraySelected
		{
			None = 0,
			Ctrl = 1,
			Alt = 2
		};
		private static string[][] keyArray = {
			new string[] {"F1", "F2", "F3", "F4", "F5", "F6", "F7", "F8", "F9", "F10", "F11", "F12",},
			new string[] {"F1", "F2", "F3", "F4", "F5", "F6", "F7", "F8", "F9", "F10", "F11", "F12",
			"1", "2", "3", "4", "5", "6", "7", "8", "9", "0",
			"A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M",
			"N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z"},
			new string[] {"F1", "F2", "F3", "F4", "F5", "F6", "F7", "F8", "F9", "F10", "F11", "F12",
			"1", "2", "3", "4", "5", "6", "7", "8", "9", "0",
			"A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M",
			"N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z"}
		};
		#endregion

		#region Constructor
		public MacroForm()
		{
			InitializeComponent();

			macroConfig = new MacroConfig();
			try {
				macroConfig.LoadConfig();
			}
			catch {}

			macrosArray = macroConfig.Macros;
			SetComboBox(keyArraySelected.None);
			PopulateListView(keyArraySelected.None);
		}
		#endregion

		#region Windows Forms Designer generated code
		/// <summary>
		/// This method is required for Windows Forms designer support.
		/// Do not change the method contents inside the source code editor. The Forms designer might
		/// not be able to load this method if it was changed manually.
		/// </summary>
		private void InitializeComponent() {
			this.comboBoxKey = new System.Windows.Forms.ComboBox();
			this.radioButtonCtrl = new System.Windows.Forms.RadioButton();
			this.listViewKeys = new System.Windows.Forms.ListView();
			this.buttonCancel = new System.Windows.Forms.Button();
			this.radioButtonAlt = new System.Windows.Forms.RadioButton();
			this.columnHeader2 = new System.Windows.Forms.ColumnHeader();
			this.columnHeader1 = new System.Windows.Forms.ColumnHeader();
			this.label2 = new System.Windows.Forms.Label();
			this.radioButtonNone = new System.Windows.Forms.RadioButton();
			this.textBoxText = new System.Windows.Forms.TextBox();
			this.buttonChange = new System.Windows.Forms.Button();
			this.groupBox1 = new System.Windows.Forms.GroupBox();
			this.label1 = new System.Windows.Forms.Label();
			this.buttonOkay = new System.Windows.Forms.Button();
			this.groupBox1.SuspendLayout();
			this.SuspendLayout();
			// 
			// comboBoxKey
			// 
			this.comboBoxKey.Location = new System.Drawing.Point(48, 168);
			this.comboBoxKey.Name = "comboBoxKey";
			this.comboBoxKey.Size = new System.Drawing.Size(152, 24);
			this.comboBoxKey.TabIndex = 8;
			this.comboBoxKey.Text = "Please Choose";
			// 
			// radioButtonCtrl
			// 
			this.radioButtonCtrl.Location = new System.Drawing.Point(128, 24);
			this.radioButtonCtrl.Name = "radioButtonCtrl";
			this.radioButtonCtrl.Size = new System.Drawing.Size(48, 24);
			this.radioButtonCtrl.TabIndex = 1;
			this.radioButtonCtrl.Text = "Ctrl";
			this.radioButtonCtrl.Click += new System.EventHandler(this.Handle_RadioButtons);
			// 
			// listViewKeys
			// 
			this.listViewKeys.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
						this.columnHeader1,
						this.columnHeader2});
			this.listViewKeys.FullRowSelect = true;
			this.listViewKeys.HeaderStyle = System.Windows.Forms.ColumnHeaderStyle.Nonclickable;
			this.listViewKeys.Location = new System.Drawing.Point(0, 0);
			this.listViewKeys.MultiSelect = false;
			this.listViewKeys.Name = "listViewKeys";
			this.listViewKeys.Size = new System.Drawing.Size(288, 97);
			this.listViewKeys.TabIndex = 0;
			this.listViewKeys.View = System.Windows.Forms.View.Details;
			this.listViewKeys.SelectedIndexChanged += new System.EventHandler(this.SelectedIndex_Changed);
			// 
			// buttonCancel
			// 
			this.buttonCancel.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.buttonCancel.Location = new System.Drawing.Point(160, 240);
			this.buttonCancel.Name = "buttonCancel";
			this.buttonCancel.TabIndex = 6;
			this.buttonCancel.Text = "&Cancel";
			this.buttonCancel.Click += new System.EventHandler(this.ButtonCancel_Clicked);
			// 
			// radioButtonAlt
			// 
			this.radioButtonAlt.Location = new System.Drawing.Point(224, 24);
			this.radioButtonAlt.Name = "radioButtonAlt";
			this.radioButtonAlt.Size = new System.Drawing.Size(48, 24);
			this.radioButtonAlt.TabIndex = 2;
			this.radioButtonAlt.Text = "Alt";
			this.radioButtonAlt.Click += new System.EventHandler(this.Handle_RadioButtons);
			// 
			// columnHeader2
			// 
			this.columnHeader2.Text = "Text";
			this.columnHeader2.Width = 224;
			// 
			// columnHeader1
			// 
			this.columnHeader1.Text = "Key";
			// 
			// label2
			// 
			this.label2.Location = new System.Drawing.Point(8, 168);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(40, 23);
			this.label2.TabIndex = 7;
			this.label2.Text = "Key";
			// 
			// radioButtonNone
			// 
			this.radioButtonNone.Checked = true;
			this.radioButtonNone.Location = new System.Drawing.Point(24, 24);
			this.radioButtonNone.Name = "radioButtonNone";
			this.radioButtonNone.Size = new System.Drawing.Size(56, 24);
			this.radioButtonNone.TabIndex = 0;
			this.radioButtonNone.TabStop = true;
			this.radioButtonNone.Text = "None";
			this.radioButtonNone.Click += new System.EventHandler(this.Handle_RadioButtons);
			// 
			// textBoxText
			// 
			this.textBoxText.Location = new System.Drawing.Point(48, 200);
			this.textBoxText.Name = "textBoxText";
			this.textBoxText.Size = new System.Drawing.Size(152, 24);
			this.textBoxText.TabIndex = 3;
			this.textBoxText.Text = "";
			// 
			// buttonChange
			// 
			this.buttonChange.Location = new System.Drawing.Point(208, 184);
			this.buttonChange.Name = "buttonChange";
			this.buttonChange.TabIndex = 4;
			this.buttonChange.Text = "C&hange";
			this.buttonChange.Click += new System.EventHandler(this.ButtonChange_Clicked);
			// 
			// groupBox1
			// 
			this.groupBox1.Controls.Add(this.radioButtonAlt);
			this.groupBox1.Controls.Add(this.radioButtonCtrl);
			this.groupBox1.Controls.Add(this.radioButtonNone);
			this.groupBox1.Location = new System.Drawing.Point(0, 104);
			this.groupBox1.Name = "groupBox1";
			this.groupBox1.Size = new System.Drawing.Size(288, 56);
			this.groupBox1.TabIndex = 1;
			this.groupBox1.TabStop = false;
			this.groupBox1.Text = "Modifier";
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(8, 200);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(40, 23);
			this.label1.TabIndex = 2;
			this.label1.Text = "&Text";
			// 
			// buttonOkay
			// 
			this.buttonOkay.DialogResult = System.Windows.Forms.DialogResult.OK;
			this.buttonOkay.Location = new System.Drawing.Point(48, 240);
			this.buttonOkay.Name = "buttonOkay";
			this.buttonOkay.TabIndex = 5;
			this.buttonOkay.Text = "&Okay";
			this.buttonOkay.Click += new System.EventHandler(this.ButtonOkay_Clicked);
			// 
			// MacroForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(7, 17);
			this.ClientSize = new System.Drawing.Size(292, 271);
			this.Controls.Add(this.comboBoxKey);
			this.Controls.Add(this.label2);
			this.Controls.Add(this.buttonCancel);
			this.Controls.Add(this.buttonOkay);
			this.Controls.Add(this.buttonChange);
			this.Controls.Add(this.textBoxText);
			this.Controls.Add(this.label1);
			this.Controls.Add(this.groupBox1);
			this.Controls.Add(this.listViewKeys);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.Name = "MacroForm";
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterParent;
			this.Text = "Macro Configuration";
			this.groupBox1.ResumeLayout(false);
			this.ResumeLayout(false);
		}
		#endregion

		#region Click events
		void ButtonOkay_Clicked(object sender, System.EventArgs e)
		{
			macroConfig.Macros = macrosArray;
			macroConfig.SaveConfig();
		}
		
		void ButtonCancel_Clicked(object sender, System.EventArgs e)
		{
			this.Close();
		}

		void ButtonChange_Clicked(object sender, System.EventArgs e)
		{
			string selectedItem = (string)this.comboBoxKey.SelectedItem;
			if(selectedItem.Equals("Please Choose") || this.textBoxText.Text.Length == 0)
			{
				MessageBox.Show("A key or macro text has not been entered", "Required", MessageBoxButtons.OK, MessageBoxIcon.Warning);
				return;
			}

			if(this.listViewKeys.SelectedItems.Count > 0)
			{
				Macro mc = (this.textBoxText.Tag != null ? (Macro)this.textBoxText.Tag : new Macro());
				if(this.textBoxText.Tag != null)
				{
					macrosArray.Remove(mc);
				}

				mc.MacroString = this.textBoxText.Text;
				if(this.radioButtonNone.Checked)
				{
					mc.Modifier = (int)keyArraySelected.None;
				}
				else if(this.radioButtonCtrl.Checked)
				{
					mc.Modifier = (int)keyArraySelected.Ctrl;
				}
				else if(this.radioButtonAlt.Checked)
				{
					mc.Modifier = (int)keyArraySelected.Alt;
				}

				this.listViewKeys.Items.Remove(this.listViewKeys.SelectedItems[0]);
				ListViewItem lvi = new ListViewItem(new string[] {
				                                    	mc.KeyString,
				                                    	mc.MacroString
				                                    });
				lvi.Tag = mc;
				macrosArray.Add(mc);
				this.listViewKeys.Items.Add(lvi);
				this.listViewKeys.Sort();
			}
			else
			{
				Macro mc = new Macro();
				mc.KeyString = selectedItem;
				mc.MacroString = this.textBoxText.Text;
				if(this.radioButtonNone.Checked)
				{
					mc.Modifier = (int)keyArraySelected.None;
				}
				else if(this.radioButtonCtrl.Checked)
				{
					mc.Modifier = (int)keyArraySelected.Ctrl;
				}
				else if(this.radioButtonAlt.Checked)
				{
					mc.Modifier = (int)keyArraySelected.Alt;
				}

				foreach(ListViewItem lvit in this.listViewKeys.Items)
				{
					if(lvit.Text.Equals(mc.KeyString))
					{
						DialogResult result = MessageBox.Show("Key '" + mc.KeyString + "' already exists.  Replace?", "Whoops", MessageBoxButtons.YesNo);
						if(result == DialogResult.No)
						{
							return;
						}
						else
						{
							Macro tmc = (lvit.Tag != null ? (Macro)lvit.Tag : new Macro());
							macrosArray.Remove(tmc);
							this.listViewKeys.Items.Remove(lvit);
							break;
						}
					}
				}
				ListViewItem lvi = new ListViewItem(new string[] {
				                                    	mc.KeyString,
				                                    	mc.MacroString
				                                    });
				lvi.Tag = mc;
				macrosArray.Add(mc);
				this.listViewKeys.Items.Add(lvi);
				this.listViewKeys.Sort();
				
				this.comboBoxKey.Text = "Please Choose";
				this.textBoxText.Text = String.Empty;
				this.textBoxText.Tag = null;
			}
		}

		void Handle_RadioButtons(object sender, System.EventArgs e)
		{
			if(sender == this.radioButtonNone)
			{
				SetComboBox(keyArraySelected.None);
				PopulateListView(keyArraySelected.None);
			}
			else if(sender == this.radioButtonCtrl)
			{
				SetComboBox(keyArraySelected.Ctrl);
				PopulateListView(keyArraySelected.Ctrl);
			}
			else if(sender == this.radioButtonAlt)
			{
				SetComboBox(keyArraySelected.Alt);
				PopulateListView(keyArraySelected.Alt);
			}
			this.comboBoxKey.Text = "Please Choose";
			this.textBoxText.Text = String.Empty;
			this.textBoxText.Tag = null;
		}

		private void SelectedIndex_Changed(object sender, System.EventArgs e)
		{
			if(this.listViewKeys.SelectedItems.Count > 0)
			{
				Macro mc = (Macro)this.listViewKeys.SelectedItems[0].Tag;

				this.comboBoxKey.Text = mc.KeyString;
				this.textBoxText.Text = mc.MacroString;
				this.textBoxText.Tag = mc;
			}
		}
		#endregion

		#region Utility methods
		private void SetComboBox(keyArraySelected ks)
		{
			this.comboBoxKey.Items.Clear();
			foreach(string key in keyArray[(int)ks])
			{
				this.comboBoxKey.Items.Add(key);
			}
		}

		private void PopulateListView(keyArraySelected ks)
		{
			this.listViewKeys.Items.Clear();
			foreach(Macro mac in macroConfig.Macros)
			{
				if(mac.Modifier == (int)ks)
				{
					ListViewItem lvi = new ListViewItem(new string[] {mac.KeyString, mac.MacroString});
					lvi.Tag = mac;
					this.listViewKeys.Items.Add(lvi);
				}
			}
		}
		#endregion
	}
}
