/*
Copyright (c) 2004,2005, Deryk Robosson <deryk@0x0a.com>
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
    * Neither the name of the Deryk Robosson nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

using System;
using System.IO;
using System.Drawing;
using System.Collections;
using System.Reflection;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;

namespace EvoTemplate
{
	/// <summary>
	/// EvoTemplate Plugins Form dialog
	/// </summary>
	public class PluginsForm : System.Windows.Forms.Form
	{
		#region Private Variables
		private System.ComponentModel.IContainer components;
		private System.Windows.Forms.ColumnHeader columnHeaderVersion;
		private System.Windows.Forms.Button buttonConfigure;
		private System.Windows.Forms.ListView listViewPlugins;
		private System.Windows.Forms.Button buttonEnable;
		private System.Windows.Forms.ColumnHeader columnHeaderDescription;
		private System.Windows.Forms.Button buttonDisable;
		private System.Windows.Forms.Button buttonOkay;
		private System.Windows.Forms.ColumnHeader columnHeaderName;
		private System.Windows.Forms.ToolTip tip;
		private System.Windows.Forms.Button buttonCancel;
		private System.Windows.Forms.ColumnHeader columnHeaderStatus;
		private System.Windows.Forms.Label labelPlugins;
		private System.Windows.Forms.Button buttonRefresh;
		private string directory;
		private ArrayList pi_array;
		private ArrayList origlist;
		#endregion

		#region Default Constructor
		public PluginsForm()
		{
			InitVars();

			RefreshListFromFiles();
		}
		#endregion

		#region Overloaded Constructor
		public PluginsForm(ArrayList al)
		{
			InitVars();

			this.origlist = new ArrayList(al);
//			this.pi_array = new ArrayList(al);
			RefreshListFromArray();
		}
		#endregion

		#region InitVars
		private void InitVars()
		{
			InitializeComponent();
			InitTips();

			directory = Directory.GetCurrentDirectory() + @"\plugins\";
			pi_array = new ArrayList();
		}
		#endregion

		#region Dispose
		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}
		#endregion

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent() {
			this.components = new System.ComponentModel.Container();
			this.buttonRefresh = new System.Windows.Forms.Button();
			this.labelPlugins = new System.Windows.Forms.Label();
			this.columnHeaderStatus = new System.Windows.Forms.ColumnHeader();
			this.buttonCancel = new System.Windows.Forms.Button();
			this.tip = new System.Windows.Forms.ToolTip(this.components);
			this.columnHeaderName = new System.Windows.Forms.ColumnHeader();
			this.buttonOkay = new System.Windows.Forms.Button();
			this.buttonDisable = new System.Windows.Forms.Button();
			this.columnHeaderDescription = new System.Windows.Forms.ColumnHeader();
			this.buttonEnable = new System.Windows.Forms.Button();
			this.listViewPlugins = new System.Windows.Forms.ListView();
			this.buttonConfigure = new System.Windows.Forms.Button();
			this.columnHeaderVersion = new System.Windows.Forms.ColumnHeader();
			this.SuspendLayout();
			// 
			// buttonRefresh
			// 
			this.buttonRefresh.Location = new System.Drawing.Point(172, 184);
			this.buttonRefresh.Name = "buttonRefresh";
			this.buttonRefresh.TabIndex = 3;
			this.buttonRefresh.Text = "&Refresh";
			this.buttonRefresh.Click += new System.EventHandler(this.HandleButtons);
			// 
			// labelPlugins
			// 
			this.labelPlugins.Location = new System.Drawing.Point(8, 8);
			this.labelPlugins.Name = "labelPlugins";
			this.labelPlugins.Size = new System.Drawing.Size(120, 23);
			this.labelPlugins.TabIndex = 5;
			this.labelPlugins.Text = "Available Plugins";
			// 
			// columnHeaderStatus
			// 
			this.columnHeaderStatus.Text = "Status";
			this.columnHeaderStatus.Width = 65;
			// 
			// buttonCancel
			// 
			this.buttonCancel.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.buttonCancel.Location = new System.Drawing.Point(192, 216);
			this.buttonCancel.Name = "buttonCancel";
			this.buttonCancel.TabIndex = 7;
			this.buttonCancel.Text = "&Cancel";
			// 
			// columnHeaderName
			// 
			this.columnHeaderName.Text = "Name";
			this.columnHeaderName.Width = 80;
			// 
			// buttonOkay
			// 
			this.buttonOkay.DialogResult = System.Windows.Forms.DialogResult.OK;
			this.buttonOkay.Location = new System.Drawing.Point(64, 216);
			this.buttonOkay.Name = "buttonOkay";
			this.buttonOkay.TabIndex = 6;
			this.buttonOkay.Text = "&Okay";
			// 
			// buttonDisable
			// 
			this.buttonDisable.Enabled = false;
			this.buttonDisable.Location = new System.Drawing.Point(92, 184);
			this.buttonDisable.Name = "buttonDisable";
			this.buttonDisable.TabIndex = 2;
			this.buttonDisable.Text = "&Disable";
			this.buttonDisable.Click += new System.EventHandler(this.HandleButtons);
			// 
			// columnHeaderDescription
			// 
			this.columnHeaderDescription.Text = "Description";
			this.columnHeaderDescription.Width = 120;
			// 
			// buttonEnable
			// 
			this.buttonEnable.Enabled = false;
			this.buttonEnable.Location = new System.Drawing.Point(12, 184);
			this.buttonEnable.Name = "buttonEnable";
			this.buttonEnable.TabIndex = 1;
			this.buttonEnable.Text = "&Enable";
			this.buttonEnable.Click += new System.EventHandler(this.HandleButtons);
			// 
			// listViewPlugins
			// 
			this.listViewPlugins.Alignment = System.Windows.Forms.ListViewAlignment.Default;
			this.listViewPlugins.AllowColumnReorder = true;
			this.listViewPlugins.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
						this.columnHeaderName,
						this.columnHeaderVersion,
						this.columnHeaderDescription,
						this.columnHeaderStatus});
			this.listViewPlugins.FullRowSelect = true;
			this.listViewPlugins.GridLines = true;
			this.listViewPlugins.LabelWrap = false;
			this.listViewPlugins.Location = new System.Drawing.Point(8, 32);
			this.listViewPlugins.MultiSelect = false;
			this.listViewPlugins.Name = "listViewPlugins";
			this.listViewPlugins.Size = new System.Drawing.Size(320, 136);
			this.listViewPlugins.Sorting = System.Windows.Forms.SortOrder.Ascending;
			this.listViewPlugins.TabIndex = 0;
			this.listViewPlugins.View = System.Windows.Forms.View.Details;
			this.listViewPlugins.SelectedIndexChanged += new System.EventHandler(this.ListViewPluginsSelectedIndexChanged);
			// 
			// buttonConfigure
			// 
			this.buttonConfigure.Enabled = false;
			this.buttonConfigure.Location = new System.Drawing.Point(252, 184);
			this.buttonConfigure.Name = "buttonConfigure";
			this.buttonConfigure.TabIndex = 4;
			this.buttonConfigure.Text = "&Configure";
			this.buttonConfigure.Click += new System.EventHandler(this.HandleButtons);
			// 
			// columnHeaderVersion
			// 
			this.columnHeaderVersion.Text = "Version";
			this.columnHeaderVersion.Width = 50;
			// 
			// PluginsForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(7, 17);
			this.ClientSize = new System.Drawing.Size(338, 247);
			this.Controls.Add(this.buttonCancel);
			this.Controls.Add(this.buttonOkay);
			this.Controls.Add(this.labelPlugins);
			this.Controls.Add(this.buttonConfigure);
			this.Controls.Add(this.buttonRefresh);
			this.Controls.Add(this.buttonDisable);
			this.Controls.Add(this.buttonEnable);
			this.Controls.Add(this.listViewPlugins);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.Name = "PluginsForm";
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterParent;
			this.Text = "Plugins";
			this.ResumeLayout(false);
		}
		#endregion

		#region InitTips
		// <summary>
		// Set the tooltips for the objects on this form
		// </summary>
		private void InitTips()
		{
			tip.Active = true;
			tip.SetToolTip(listViewPlugins, "Available plugins");
			tip.SetToolTip(buttonEnable, "Enable plugin");
			tip.SetToolTip(buttonDisable, "Disable plugin");
			tip.SetToolTip(buttonRefresh, "Refresh plugins list");
			tip.SetToolTip(buttonConfigure, "Configure selected plugin");
		}
		#endregion

		#region HandleButtons
		// <summary>
		// Handle user input from the buttons on this form
		// </summary>
		private void HandleButtons(object sender, System.EventArgs e)
		{
			PluginInfo pi = GetSelectedPlugin();
			ListViewItem lvi = GetSelectedItem();

			if(sender == buttonEnable) {
				if(pi != null) {
					buttonDisable.Enabled = true;
					buttonEnable.Enabled = false;
					pi.Enabled = true;
					if(pi.Configurable) {
						buttonConfigure.Enabled = true;
					}

					// remove the item from the list and the array
					listViewPlugins.Items.Remove(lvi);
					pi_array.Remove(pi);
					origlist.Remove(pi);

					// readd them
					string[] items = new string[] {
							pi.Name,
							pi.Version,
							pi.Description,
							"Enabled"};
					ListViewItem item = new ListViewItem(items);
					item.Tag = pi;
					listViewPlugins.Items.Add(item);
					pi_array.Add(pi);
					origlist.Add(pi);
				}
			}
			else if(sender == buttonDisable) {
				if(pi != null) {
					buttonDisable.Enabled = false;
					buttonEnable.Enabled = true;
					if(pi.Configurable) {
						buttonConfigure.Enabled = false;
					}

					// remove the item from the list and the array
					listViewPlugins.Items.Remove(lvi);
					pi_array.Remove(pi);
					origlist.Remove(pi);

					// readd them
					string[] items = new string[] {
							pi.Name,
							pi.Version,
							pi.Description,
							"Disabled"};
					ListViewItem item = new ListViewItem(items);
					item.Tag = pi;
					listViewPlugins.Items.Add(item);
					pi_array.Add(pi);
					origlist.Add(pi);
				}
			}
			else if(sender == buttonRefresh) {
				this.listViewPlugins.Items.Clear();
				RefreshListFromFiles();
			}
			else if(sender == buttonConfigure) {
				if(pi != null) {
					if(pi.Configurable) {
						pi.Version = (string)pi.ObjectType.InvokeMember("Configure",
												BindingFlags.Default | BindingFlags.InvokeMethod,
												null,
												pi.BaseObject,
												null);
					}
				}
			}
		}
		#endregion

		#region InterrogatePlugin
		// <summary>
		// Obtain information about a plugin and return
		// a PluginInfo object if successful
		// </summary>
		private PluginInfo InterrogatePlugin(string plugin)
		{
			PluginInfo pi = new PluginInfo();

			try {
				Assembly assembly = Assembly.LoadFrom(plugin);

				foreach(Type type in assembly.GetTypes()) {
					if(type.IsClass == true) {
						if(type.GetInterface("EvoTemplate.IPlugin") == null) {
							continue;
						}
    
						pi.ObjectType = type;
						pi.BaseObject = Activator.CreateInstance (type);
						pi.Configurable = (bool)pi.ObjectType.InvokeMember("Configurable",
												BindingFlags.Default | BindingFlags.GetProperty,
												null,
												pi.BaseObject,
												null);

						pi.Description = (string)pi.ObjectType.InvokeMember("Description",
												BindingFlags.Default | BindingFlags.GetProperty,
												null,
												pi.BaseObject,
												null);

						pi.Filename = (string)pi.ObjectType.InvokeMember("Filename",
												BindingFlags.Default | BindingFlags.GetProperty,
												null,
												pi.BaseObject,
												null);

						pi.Name = (string)pi.ObjectType.InvokeMember("Name",
												BindingFlags.Default | BindingFlags.GetProperty,
												null,
												pi.BaseObject,
												null);

						pi.Version = (string)pi.ObjectType.InvokeMember("Version",
												BindingFlags.Default | BindingFlags.GetProperty,
												null,
												pi.BaseObject,
												null);

						pi.MenuName = (string)pi.ObjectType.InvokeMember("MenuName",
												BindingFlags.Default | BindingFlags.GetProperty,
												null,
												pi.BaseObject,
												null);
					}
				}
			}
			catch(Exception e) {
				MessageBox.Show(e.Message);
				pi = null;
			}

			return pi;
		}
		#endregion

		#region FindPlugin name
		// <summary>
		// returns the PluginInfo object for the supplied
		// name if found in the internal plugin ArrayList
		// </summary>
		private PluginInfo FindPlugin(string name)
		{
			foreach(PluginInfo pi in this.pi_array) {
				if(pi.Filename.Equals(name)) {
					return pi;
				}
			}
			return null;
		}
		#endregion

		#region FindPlugin ArrayList
		/// <summary>
		/// returns the PluginInfo object for the supplied
		/// name in the provided ArrayList
		/// </summary>
		private PluginInfo FindPlugin(ArrayList al, string name)
		{
			foreach(PluginInfo pi in al) {
				if(pi.Name.Equals(name)) {
					return pi;
				}
			}
			return null;
		}
		#endregion

		#region GetSelectedPlugin
		/// <summary>
		/// Returns the currently active selected items plugin
		/// </summary>
		private PluginInfo GetSelectedPlugin()
		{
			ListView.SelectedListViewItemCollection si = this.listViewPlugins.SelectedItems;
			//GetSelectedItems();
			PluginInfo pi = null;

			if(si.Count > 0) {
				pi = (PluginInfo)si[0].Tag;
			}

			return pi;
		}
		#endregion

		#region GetSelectedItem
		/// <summary>
		/// Returns the currently selected listview items
		/// </summary>
		private ListViewItem GetSelectedItem()
		{
			ListView.SelectedListViewItemCollection si = this.listViewPlugins.SelectedItems;
			ListViewItem lvi = null;

			if(si.Count > 0) {
				lvi = (ListViewItem)si[0];
			}

			return lvi;
		}
		#endregion

		#region Selected Index Changed
		// <summary>
		// When a selection on the listview changes, we
		// will have to alter buttons enabled/disabled
		// and set the currently selected PluginInfo
		// </summary>
		void ListViewPluginsSelectedIndexChanged(object sender, System.EventArgs e)
		{
			PluginInfo pi = GetSelectedPlugin();

			if(pi != null) {
				if(pi.Enabled) {
					this.buttonEnable.Enabled = false;
					this.buttonDisable.Enabled = true;
				}
				else {
					this.buttonEnable.Enabled = true;
					this.buttonDisable.Enabled = false;
				}
					
				if(pi.Configurable) {
					this.buttonConfigure.Enabled = true;
				}
				else {
					this.buttonConfigure.Enabled = false;
				}
			}
		}
		#endregion

		#region RefreshListFromFiles
		/// <summary>Refresh the plugin listview</summary>
		private void RefreshListFromFiles()
		{
			// keep working copy of original list, then clear
			ArrayList oldlist = new ArrayList(this.origlist);
			this.pi_array.Clear();

			// refresh plugin list
			if(Directory.Exists(this.directory)) {
				DirectoryInfo di = new DirectoryInfo(this.directory);
				FileInfo[] files = di.GetFiles("*.dll");
				if(files != null) {
					foreach(FileInfo file in files) {
						PluginInfo pi = InterrogatePlugin(file.FullName);
						if(pi != null) {
							string endi = "Disabled";
							PluginInfo pifind = FindPlugin(oldlist, pi.Name);
							if(pifind != null) {
								if(pifind.Enabled) {
									endi = "Enabled";
									pi.Enabled = true;
Console.WriteLine("Name: {0} Enabled: {1}", pifind.Name, pi.Enabled.ToString());
								}
								else {
									pi.Enabled = false;
Console.WriteLine("Name: {0} Enabled: {1}", pifind.Name, pi.Enabled.ToString());
								}
							}
							string[] items = new string[] {
								pi.Name,
								pi.Version,
								pi.Description,
								endi};
							pi.Filename = file.Name;
							ListViewItem item = new ListViewItem(items);
							item.Tag = pi;
							listViewPlugins.Items.Add(item);
							pi_array.Add(pi);
						}
					}
				}
			}
			else {
				if(MessageBox.Show("Plugin directory is missing. Create?", "Whoops",
				                   MessageBoxButtons.OKCancel, MessageBoxIcon.Question) 
				                   == DialogResult.OK) {
					try {
						DirectoryInfo di = Directory.CreateDirectory(this.directory);
					}
					catch(Exception ex) {
						MessageBox.Show(ex.Message);
					}
				}
			}
		}
		#endregion

		#region RefreshListFromArray
		private void RefreshListFromArray()
		{
			this.listViewPlugins.Items.Clear();
			foreach(PluginInfo pi in this.origlist) {
				PluginInfo pi2 = InterrogatePlugin(this.directory + "/" + pi.Filename);
				string[] items = new string[] {
						pi.Name,
						pi.Version,
						pi.Description,
						(pi.Enabled ? "Enabled" : "Disabled")};
				ListViewItem item = new ListViewItem(items);
				item.Tag = pi;
				listViewPlugins.Items.Add(item);
			}
		}
		#endregion

		#region PluginInfoArray
		public ArrayList PluginInfoArray
		{
			get {
				return pi_array;
			}
			set {
				this.pi_array = value;
				this.origlist = value;
				RefreshListFromArray();
			}
		}
		#endregion
	}
}
