/*
Copyright (c) 2004,2005, Deryk Robosson <deryk@0x0a.com>
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
    * Neither the name of the Deryk Robosson nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

using System;
using System.Net;
using System.IO;

namespace EvoTemplate
{
	/// <summary>
	/// Description of UpdateDetect.	
	/// </summary>
	public class UpdateDetect
	{
		#region Private Variables
		private WebClient client;
		private int version;
		private int revision;
		private string uri;
		private string exceptionError;
		#endregion

		#region Default Constructor
		public UpdateDetect(string server)
		{
			client = new WebClient();
			version = 0;
			revision = 0;
			uri = "http://" + server + "/versioncheck.php";
			exceptionError = null;
		}
		#endregion

		#region Overloaded Check method
		/// <summary>Connects to the default URI</summary>
		/// <returns>bool, true for success, false for failure</returns>
		public bool Check() {
			return Check(uri);
		}
		#endregion

		#region Overloaded Check method
		/// <summary>Connects to the specified URI to check current version and revision.</summary>
		/// <param name="uri">URI to check</param>
		/// <returns>bool, true for success, false for failure.</returns>
		public bool Check(string strUri)
		{
			bool retval = false;

			if(uri == null) {
				throw new System.ArgumentNullException();
			}

			uri = strUri;

			try {
				client.Headers.Add ("user-agent", "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; .NET CLR 1.0.3705;)");

		        Stream data = client.OpenRead(uri);
    		    StreamReader reader = new StreamReader(data);
        		version = Convert.ToInt32(reader.ReadLine());
				revision = Convert.ToInt32(reader.ReadLine());
	        	data.Close();
    	    	reader.Close();
				
				retval = true;
			}
			catch(Exception e) {
				exceptionError = e.Message;
				retval = false;
			}
			
			return(retval);
		}
		#endregion
		
		#region VersionString property
		/// <summary>Gets software version number</summary>
		/// <returns>int</returns>
		public string VersionString
		{
			get {
				return version.ToString() + "." + revision.ToString();
			}
		}
		#endregion

		#region Version property
		/// <summary>Gets software version number</summary>
		/// <returns>int</returns>
		public int Version
		{
			get {
				return version;
			}
		}
		#endregion
		
		#region Revision property
		/// <summary>Gets software revision number</summary>
		/// <returns>int</returns>
		public int Revision
		{
			get {
				return revision;
			}
		}
		#endregion
		
		#region Error property
		/// <summary>Gets an error if check fails</summary>
		/// <returns>string</returns>
		public string Error
		{
			get {
				return exceptionError;
			}
		}
		#endregion
	}
}
